-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THCore = {}
local THCore_mt = Class(THCore)
THCore.SPEC_TYPE = {
VEHICLE   = "vehicle",
PLACEABLE = "placeable"
}
function THCore.new(customMt)
customMt = Utils.getNoNil(customMt, THCore_mt)
local newEnv = setmetatable({}, customMt)
if newEnv ~= nil then
local missionInfo = g_mpLoadingScreen.missionInfo
local missionDynamicInfo = g_mpLoadingScreen.missionDynamicInfo
newEnv.isServer = g_server ~= nil
newEnv.isClient = g_client ~= nil
newEnv.actionEventIds = {}
newEnv.isMultiplayer  = missionDynamicInfo.isMultiplayer
newEnv.isEnabled      = true
newEnv.modInfo = nil
newEnv.modName = g_currentModName
newEnv.modPath = g_currentModDirectory
newEnv.modList = missionDynamicInfo.mods
newEnv.modSettingsPath = g_currentModSettingsDirectory
newEnv.sdkPath = newEnv.modPath.."sdk/"
createFolder(newEnv.modSettingsPath)
createFolder(newEnv.modSettingsPath.."maps/")
newEnv.loadedMods = {
byId    = {},
byIndex = {}
}
local mapInfo = missionInfo.map
local mapName = mapInfo.id
if mapInfo.isModMap then
if THUtils.getNoNil(mapInfo.customEnvironment, "") ~= "" then
mapName = mapInfo.customEnvironment
end
end
if THUtils.getNoNil(mapName, "") ~= "" then
newEnv.mapName = mapName
newEnv.mapSettingsPath = newEnv.modSettingsPath.."maps/"..mapName.."/"
createFolder(newEnv.mapSettingsPath)
end
newEnv.mapInfo = mapInfo
newEnv.mapIsMod = mapInfo.isModMap
newEnv.mapXMLFile = mapInfo.mapXMLFilename
newEnv.mapModInfo = nil
if newEnv.mapIsMod then
newEnv.mapPath = mapInfo.baseDirectory
end
newEnv.settingsXMLFile = newEnv.modSettingsPath.."settings.xml"
newEnv.settings = {
byId       = {},
byIndex    = {},
byCategory = {},
idToIndex  = {}
}
newEnv.SETTING = newEnv.settings.idToIndex
newEnv.settingCategories = {
byId      = {},
byIndex   = {},
idToIndex = {}
}
newEnv.SETTING_CATEGORY = newEnv.settingCategories.idToIndex
newEnv.specializations = {
byId      = {},
byType    = {},
byIndex   = {},
idToIndex = {}
}
newEnv.SPECIALIZATION = newEnv.specializations.idToIndex
for _, typeName in pairs(THCore.SPEC_TYPE) do
newEnv.specializations.byType[typeName] = {}
end
end
return newEnv
end
function THCore:call(target, arg1, ...)
local function getRetValues(pSuccess, p1, ...)
if pSuccess then
return p1, ...
end
self.isEnabled = false
if type(p1) == "string" then
THUtils.errorMsg(nil, p1)
else
printCallstack()
end
end
if self.isEnabled then
if type(target) == "string" then
return getRetValues(THUtils.pcall(self, target, arg1, ...))
end
return getRetValues(THUtils.pcall(target, arg1, ...))
end
end
function THCore:setProtectedHook(srcClass, srcFuncKey, tgtClass, tgtFuncKey, useSelf)
useSelf = THUtils.getNoNil(useSelf, true)
if type(srcClass) == "string" then
local globalEnv = THUtils.getGlobalEnv()
srcClass = globalEnv[srcClass]
end
if tgtFuncKey == nil then
if type(srcFuncKey) == "string" then
tgtFuncKey = "hook_"..srcFuncKey
end
end
local srcFunc = srcClass[srcFuncKey]
local tgtFunc = tgtClass[tgtFuncKey]
if  THUtils.assert(type(srcFunc) == "function", true, "Invalid source function %q", srcFuncKey)
and THUtils.assert(type(tgtFunc) == "function", true, "Invalid target function %q", tgtFuncKey)
then
rawset(srcClass, srcFuncKey, function(p1, ...)
if self.isEnabled then
if useSelf then
return tgtFunc(p1, srcFunc, ...)
else
return tgtFunc(srcFunc, p1, ...)
end
else
return srcFunc(p1, ...)
end
end)
return true
end
return false
end
function THCore:createDataTable(parent, modName, clear, upValue, ...)
if parent == nil then
return
end
local tableKey = "data"
if modName == nil then
tableKey = tableKey.."."..self.modName
else
local modInfo = self:getLoadedMod(tostring(modName))
if modInfo == nil then
return
end
tableKey = tableKey.."."..modInfo.name
end
local dataTable = rawget(parent, tableKey)
if dataTable == nil or clear == true then
local customMt = nil
local oldHasEnvHooks = nil
if upValue ~= nil then
customMt = Class(upValue, THData)
end
if dataTable ~= nil and clear then
if dataTable.deleteDataTable ~= nil then
local deleteSuccess = dataTable:deleteDataTable()
if not deleteSuccess then
THUtils.errorMsg(true, "Could not delete data table")
return
end
end
oldHasEnvHooks = dataTable.hasEnvironmentHooks
end
dataTable = THData.new(parent, customMt)
rawset(parent, tableKey, dataTable)
dataTable:updateDefaultValues(true)
if not dataTable.isTableInitialized then
if dataTable.initializeDataTable ~= nil then
if dataTable:initializeDataTable(...) then
dataTable.isTableInitialized = true
else
THUtils.errorMsg(true, "Failed to initialize dataTable (%s)", tableKey)
end
end
end
if oldHasEnvHooks ~= nil then
dataTable.hasEnvironmentHooks = oldHasEnvHooks
end
if not dataTable.hasEnvironmentHooks then
if dataTable.setEnvironmentHooks ~= nil then
if dataTable:setEnvironmentHooks(parent) then
dataTable.hasEnvironmentHooks = true
else
THUtils.errorMsg(true, "Failed to set environment hooks for dataTable (%s)", tableKey)
end
end
end
end
return dataTable, parent
end
function THCore:getDataTable(parent, modName, create, upValue, ...)
if create == true then
return self:createDataTable(parent, modName, false, upValue, ...)
end
if parent == nil then
return
end
local tableKey = "data"
if modName == nil then
tableKey = tableKey.."."..self.modName
else
local modInfo = self:getLoadedMod(tostring(modName))
if modInfo == nil then
return
end
tableKey = tableKey.."."..modInfo.name
end
local dataTable = rawget(parent, tableKey)
return dataTable, parent
end
function THCore:initializeLoadedMods()
for _, modInfo in ipairs(self.modList) do
local modName = THUtils.getNoNil(modInfo.modName, "")
if modName ~= "" then
local modEntry = {
id    = modName:upper(),
info  = modInfo,
name  = modName,
path  = modInfo.modDir,
file  = modInfo.modFile,
index = #self.loadedMods.byIndex + 1,
title = modInfo.title
}
self.loadedMods.byId[modEntry.id] = modEntry
self.loadedMods.byIndex[modEntry.index] = modEntry
if modName == self.modName then
self.modInfo = modEntry
end
if self.mapIsMod and self.mapName ~= nil and modName == self.mapName then
self.mapModInfo = modEntry
end
end
end
end
function THCore:getLoadedMod(modName, verbose, raise)
local valType = type(modName)
local modInfo = nil
if valType == "string" then
modInfo = self.loadedMods.byId[modName:upper()]
elseif modName ~= nil then
verbose = true
end
raise = THUtils.getNoNil(raise, verbose)
THUtils.msgOnTrue(modInfo == nil and verbose, raise, THUtils.MSG.ARGUMENT_INVALID, "modName", modName)
return modInfo
end
function THCore:getLoadedModList(byId)
if byId == true then
return self.loadedMods.byId
end
return self.loadedMods.byIndex
end
function THCore:initializeSettings()
return true
end
function THCore:loadSettings()
if fileExists(self.settingsXMLFile) then
local baseKey = "settings"
local xmlFile = loadXMLFile("THSettings", self.settingsXMLFile)
if THUtils.getNoNil(xmlFile, 0) ~= 0 then
local idx = 0
while true do
local settingKey = string.format("%s.setting(%d)", baseKey, idx)
if not hasXMLProperty(xmlFile, settingKey) then
break
end
local settingName = getXMLString(xmlFile, settingKey.."#name")
local settingInfo = self:getSetting(settingName)
if settingInfo ~= nil then
local valIndex = getXMLInt(xmlFile, settingKey.."#value")
if valIndex ~= nil and settingInfo.values[valIndex] ~= nil then
settingInfo.currentValue = valIndex
settingInfo.changedValue = valIndex
settingInfo.savedValue = valIndex
end
end
idx = idx + 1
end
delete(xmlFile)
end
end
return true
end
function THCore:saveSettings()
local settingsList = self:getSettingsList()
local baseKey = "settings"
local xmlFile = createXMLFile("THSettings", self.settingsXMLFile, baseKey)
if THUtils.getNoNil(xmlFile, 0) ~= 0 then
for i, settingInfo in ipairs(settingsList) do
local settingKey = string.format("%s.setting(%d)", baseKey, i-1)
setXMLString(xmlFile, settingKey.."#name",  settingInfo.name)
setXMLInt(xmlFile,    settingKey.."#value", settingInfo.changedValue)
settingInfo.savedValue = settingInfo.changedValue
end
saveXMLFile(xmlFile)
delete(xmlFile)
end
return true
end
function THCore:finalizeSettings()
return true
end
function THCore:addSettingCategory(name, title, customEnv, basePath)
customEnv = THUtils.getNoNil(customEnv, self.modName, true)
basePath  = THUtils.getNoNil(basePath, self.modPath, true)
local modInfo = self:getLoadedMod(customEnv, true)
if modInfo == nil then
return
end
if  THUtils.argIsValid(type(name) == "string", "name", name)
and THUtils.argIsValid(title == nil or type(title) == "string", "title", title)
then
local categoryInfo = self:getSettingCategory(name)
if categoryInfo == nil then
categoryInfo = {
id             = name:upper(),
name           = name,
index          = #self.settingCategories.byIndex + 1,
customEnv      = modInfo.name,
settings       = {},
settingMapping = {}
}
if title == nil or title == "" then
if g_i18n:hasText("thTitle_"..categoryInfo.name, categoryInfo.customEnv) then
categoryInfo.title = g_i18n:getText("thTitle_"..categoryInfo.name, categoryInfo.customEnv)
else
categoryInfo.title = categoryInfo.name
end
else
categoryInfo.title = g_i18n:convertText(title, categoryInfo.customEnv)
end
self.settingCategories.byId[categoryInfo.id] = categoryInfo
self.settingCategories.byIndex[categoryInfo.index] = categoryInfo
self.settingCategories.idToIndex[categoryInfo.id] = categoryInfo.index
end
return categoryInfo
end
end
function THCore:getSettingCategory(target, verbose, raise)
return THUtils.getTargetInfo(self, "settingCategories", target, verbose, raise)
end
function THCore:getSettingCategoryList()
return self.settingCategories.byIndex
end
function THCore:addSetting(name, title, toolTip, category, customEnv, basePath, isCheckedOption, restartRequired)
category  = THUtils.getNoNil(category, 1)
customEnv = THUtils.getNoNil(customEnv, self.modName, true)
basePath  = THUtils.getNoNil(basePath, self.modPath, true)
isCheckedOption = THUtils.getNoNil(isCheckedOption, false)
restartRequired = THUtils.getNoNil(restartRequired, false)
local categoryInfo = self:getSettingCategory(category, true)
local modInfo = self:getLoadedMod(customEnv, true)
if categoryInfo == nil or modInfo == nil then
return
end
if  THUtils.argIsValid(type(name) == "string", "name", name)
and THUtils.argIsValid(title == nil or type(title) == "string", "title", title)
and THUtils.argIsValid(toolTip == nil or type(toolTip) == "string", "toolTip", toolTip)
and THUtils.argIsValid(not isCheckedOption or isCheckedOption == true, "isCheckedOption", isCheckedOption)
and THUtils.argIsValid(not restartRequired or restartRequired == true, "restartRequired", restartRequired)
then
local settingInfo = self:getSetting(name)
if settingInfo == nil then
settingInfo = {
id              = name:upper(),
name            = name,
index           = #self.settings.byIndex + 1,
isCheckedOption = false,
restartRequired = restartRequired,
customEnv       = modInfo.name,
values          = {},
valueTexts      = {},
valueMapping    = {},
currentValue    = 0,
changedValue    = 0,
savedValue      = 0,
}
if title == nil or title == "" then
if g_i18n:hasText("thTitle_"..settingInfo.name, settingInfo.customEnv) then
settingInfo.title = g_i18n:getText("thTitle_"..settingInfo.name, settingInfo.customEnv)
else
settingInfo.title = settingInfo.name
end
else
settingInfo.title = g_i18n:convertText(title, settingInfo.customEnv)
end
if toolTip == nil then
if g_i18n:hasText("thTooltip_"..settingInfo.name, settingInfo.customEnv) then
settingInfo.toolTip = g_i18n:getText("thTooltip_"..settingInfo.name, settingInfo.customEnv)
else
settingInfo.toolTip = ""
end
else
settingInfo.toolTip = g_i18n:convertText(toolTip, settingInfo.customEnv)
end
self.settings.byId[settingInfo.id] = settingInfo
self.settings.byIndex[settingInfo.index] = settingInfo
self.settings.idToIndex[settingInfo.id] = settingInfo.index
table.insert(categoryInfo.settings, settingInfo.index)
categoryInfo.settingMapping[settingInfo.index] = #categoryInfo.settings
if isCheckedOption then
if  self:addSettingValue(settingInfo, false, "$l10n_ui_off")
and self:addSettingValue(settingInfo, true, "$l10n_ui_on")
then
settingInfo.isCheckedOption = true
end
end
MessageType.TH_SETTING_CHANGED = THUtils.getNoNil(MessageType.TH_SETTING_CHANGED, {})
MessageType.TH_SETTING_CHANGED[settingInfo.id] = nextMessageTypeId()
settingInfo.messageTypeId = MessageType.TH_SETTING_CHANGED[settingInfo.id]
end
return settingInfo
end
end
function THCore:getSetting(target, verbose, raise)
return THUtils.getTargetInfo(self, "settings", target, verbose, raise)
end
function THCore:getSettingsList()
return self.settings.byIndex
end
function THCore:addSettingValue(setting, value, title, customEnv, basePath)
customEnv = THUtils.getNoNil(customEnv, self.modName, true)
basePath  = THUtils.getNoNil(basePath, self.modPath, true)
local settingInfo = self:getSetting(setting, true)
local modInfo = self:getLoadedMod(customEnv, true)
if settingInfo == nil or modInfo == nil then
return false
end
if  THUtils.argIsValid(value ~= nil, "value", value)
and THUtils.argIsValid(type(title) == "string" and title ~= "", "title", title)
then
if settingInfo.isCheckedOption then
THUtils.errorMsg(true, "Cannot add value to checked setting %q", settingInfo.name)
return false
end
title = g_i18n:convertText(title, customEnv)
local valueFound = false
local titleFound = false
for idx = 1, #settingInfo.values do
local chkValue = settingInfo.values[idx]
local chkTitle = settingInfo.valueTexts[idx]
if not valueFound and value == chkValue then
valueFound = true
end
if not titleFound and title == chkTitle then
titleFound = true
end
if valueFound and titleFound then
break
end
end
if valueFound then
THUtils.errorMsg(true, "Value %q already added to setting %q", value, settingInfo.name)
return false
end
if titleFound then
THUtils.errorMsg(true, "Title %q already added to setting %q", title, settingInfo.name)
return false
end
local valueIndex = #settingInfo.values + 1
settingInfo.values[valueIndex] = value
settingInfo.valueTexts[valueIndex] = title
settingInfo.valueMapping[value] = valueIndex
if valueIndex == 1 then
settingInfo.currentValue = 1
settingInfo.changedValue = 1
settingInfo.savedValue = 1
end
return true
end
return false
end
function THCore:getSettingValue(setting, getChanged, verbose, raise)
local settingInfo = self:getSetting(setting, verbose, raise)
if settingInfo ~= nil then
local valueIndex = settingInfo.currentValue
if getChanged then
valueIndex = settingInfo.changedValue
end
local retValue = settingInfo.values[valueIndex]
if retValue == nil and verbose == true then
raise = THUtils.getNoNil(raise, verbose)
THUtils.errorMsg(raise, "Setting %q, return value not found", settingInfo.name)
end
return retValue
end
end
function THCore:loadConfiguration(typeKey, xmlSchema)
if THUtils.argIsValid(THUtils.getIsType(xmlSchema, XMLSchema), "xmlSchema", xmlSchema, true) then
local typeKeyProper = THUtils.properCase(typeKey)
local typeFilename = typeKey..".xml"
local loadFunc = self["load"..typeKeyProper.."FromXMLFile"]
local sdkFilename = self.sdkPath.."xml/"..typeFilename
if fileExists(sdkFilename) then
copyFile(sdkFilename, self.modSettingsPath..typeFilename, false)
end
local dataFilename, dataPath, dataModName = self:getConfigFile(typeFilename)
if dataFilename ~= nil then
local dataXMLFile = XMLFile.loadIfExists("THModData", dataFilename, xmlSchema)
if THUtils.getNoNil(dataXMLFile, 0) ~= 0 then
loadFunc(self, dataXMLFile, dataModName, dataPath)
dataXMLFile:delete()
end
end
return true
end
return false
end
function THCore:getConfigFile(target)
local filename, basePath, modName = nil,nil,nil
if g_thModSettings ~= nil then
filename, basePath, modName = g_thModSettings:getConfigFile(target)
if filename ~= nil then
return filename, basePath, modName
end
end
if self.mapSettingsPath ~= nil then
filename = THUtils.getFilename(target, self.mapSettingsPath)
if filename ~= nil and fileExists(filename) then
return filename, self.mapSettingsPath
end
end
filename = THUtils.getFilename(target, self.modSettingsPath)
if filename ~= nil and fileExists(filename) then
return filename, self.modSettingsPath
end
end
function THCore:addSpecialization(typeName, specName, className, parentName, filename)
local modName = self.modName
local modPath = self.modPath
local specManager = nil
if type(typeName) == "string" then
typeName = THCore.SPEC_TYPE[typeName:upper()]
end
if typeName == THCore.SPEC_TYPE.VEHICLE then
specManager = g_specializationManager
elseif typeName == THCore.SPEC_TYPE.PLACEABLE then
specManager = g_placeableSpecializationManager
else
THUtils.errorMsg(true, THUtils.MSG.ARGUMENT_INVALID, "typeName", typeName)
return
end
local internalSpecName = modName.."."..specName
local specEntry = specManager:getSpecializationByName(internalSpecName)
local parentSpecEntry = nil
filename = modPath..filename
if parentName ~= nil then
parentSpecEntry = specManager:getSpecializationByName(parentName)
if parentSpecEntry == nil then
THUtils.errorMsg(true, "Parent specialization %q not found", parentName)
return
end
parentName = parentSpecEntry.name
end
if  THUtils.assert(fileExists(filename), true, "Cannot load filename %s", filename)
and THUtils.assert(specEntry == nil, true, "Specialization %q already exists", specName)
then
local specInfo = self:getSpecialization(specName)
if specInfo == nil then
specInfo = {
id    = specName:upper(),
name  = specName,
type  = typeName,
index = #self.specializations.byIndex + 1,
parentName = parentName,
parentEntry = parentSpecEntry
}
specManager:addSpecialization(specName, className, filename, modName)
specEntry = specManager:getSpecializationByName(internalSpecName)
local specClass = _G[className]
if THUtils.msgOnTrue(specEntry == nil, true, "Specialization %q was not added correctly", specName)
or THUtils.msgOnTrue(specClass == nil, true, THUtils.MSG.ARGUMENT_INVALID, "className", className)
then
return
end
specInfo.entry = specEntry
specInfo.class = specClass
self.specializations.byId[specInfo.id] = specInfo
self.specializations.byIndex[specInfo.index] = specInfo
self.specializations.idToIndex[specInfo.id] = specInfo.index
table.insert(self.specializations.byType[specInfo.type], specInfo)
end
return specInfo
end
end
function THCore:getSpecialization(target, verbose, raise)
return THUtils.getTargetInfo(self, "specializations", target, verbose, raise)
end
function THCore:getSpecializationsList()
return self.specializations.byIndex
end
function THCore:getSpecializationsByType(typeName)
if type(typeName) == "string" then
typeName = THCore.SPEC_TYPE[typeName:upper()]
end
if THUtils.argIsValid(typeName ~= nil, "typeName", typeName, true) then
return self.specializations.byType[typeName]
end
end
function THCore:getSpecTable(target, specName, modName)
if modName == nil or modName == true then
modName = self.modName
end
if modName ~= false then
specName = modName.."."..specName
end
local specTable = target["spec_"..specName]
local specData, vehicleData = nil,nil
if specTable ~= nil then
specData = self:getDataTable(specTable, modName)
vehicleData = self:getDataTable(target, modName)
end
return specTable, specData, vehicleData
end