-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THMain = {}
local THMain_mt = Class(THMain, THCore)
THMain.MAX_VALUE_BITS = 32
THMain.ACRE_FACTOR = 2.4710538146717
function THMain.new(customMt)
customMt = THUtils.getNoNil(customMt, THMain_mt)
local self = THCore.new(customMt)
if self ~= nil then
end
return self
end
function THMain:formatFillTypeUsage(value, precision, fillType)
local useAcres = g_gameSettings:getValue(GameSettings.SETTING.USE_ACRE)
local areaText, unitText = nil,nil
value = value * 10000
if useAcres then
areaText = g_i18n:getText("unit_acreShort")
value = value / THMain.ACRE_FACTOR
else
areaText = g_i18n:getText("unit_haShort")
end
if fillType ~= nil then
if g_thUnitConvert ~= nil then
local unitInfo, unitFactor = g_thUnitConvert:getFillTypeUnitData(fillType)
if unitInfo ~= nil then
value     = value * unitFactor
precision = unitInfo.precision
unitText  = unitInfo.titleShort
end
else
local fillTypeInfo = THUtils.getFillType(fillType)
if fillTypeInfo ~= nil then
unitText = fillTypeInfo.unitShort
end
end
end
if unitText == nil then
unitText = g_i18n:getVolumeUnit()
end
precision = THUtils.getNoNil(precision, 0)
precision = math.floor(math.max(0, precision))
value = THUtils.round(value, precision)
if precision > 0 then
return string.format("%0."..tostring(precision).."f %s/%s", value, unitText, areaText)
end
return string.format("%d %s/%s", value, unitText, areaText)
end
function THMain:formatWorkedArea(value, precision)
local useAcres = g_gameSettings:getValue(GameSettings.SETTING.USE_ACRE)
local areaText = nil
if useAcres then
areaText = g_i18n:getText("unit_acreShort")
value = value * THMain.ACRE_FACTOR
else
areaText = g_i18n:getText("unit_haShort")
end
precision = THUtils.getNoNil(precision, 1)
precision = math.floor(math.max(0, precision))
value = THUtils.floor(value, precision)
if precision > 0 then
return string.format("%0."..tostring(precision).."f %s", value, areaText)
end
return string.format("%d %s", value, areaText)
end
function THMain:hook_loadMapFinished(superFunc, ...)
local function appendFunc(...)
local protectedChunk = function()
if not self.cancelLoading then
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
function THMain:hook_finalizeTypes(superFunc, ...)
local function prependFunc()
local typeList = self:getTypes()
local specList = g_thMain:getSpecializationsByType(self.typeName)
for typeName, typeEntry in pairs(typeList) do
for _, specInfo in ipairs(specList) do
local internalSpecName = specInfo.entry.name
local parentName = specInfo.parentName
local specClass = specInfo.class
if parentName == nil or typeEntry.specializationsByName[parentName] ~= nil then
if  typeEntry.specializationsByName[internalSpecName] == nil
and specClass.prerequisitesPresent(typeEntry.specializations)
then
self:addSpecialization(typeName, internalSpecName)
end
end
end
end
end
g_thMain:call(prependFunc)
return superFunc(self, ...)
end
function THMain:hook_registerActionEvents(superFunc, ...)
local function appendFunc(...)
local function protectedChunk()
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
function THMain:hook_unregisterActionEvents(superFunc, ...)
local function appendFunc(...)
local function protectedChunk()
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
g_thMain = THMain.new()
if g_thMain ~= nil then
local function runScript()
local globalEnv = THUtils.getGlobalEnv()
local modName = g_thMain.modName
local modPath = g_thMain.modPath
g_thMain:initializeLoadedMods()
g_thMain:setProtectedHook("BaseMission", "loadMapFinished", THMain)
g_thMain:setProtectedHook("BaseMission", "unregisterActionEvents", THMain)
g_thMain:setProtectedHook("FSBaseMission", "registerActionEvents", THMain)
g_thMain:setProtectedHook("TypeManager", "finalizeTypes", THMain)
source(modPath.."scripts/vehicles/VHXVehicle.lua")
source(modPath.."scripts/gui/hud/VHXFillLevelsDisplay.lua")
source(modPath.."scripts/gui/hud/VHXSpeedMeterDisplay.lua")
source(modPath.."scripts/events/VHXClearFillTypeUsageEvent.lua")
source(modPath.."scripts/events/VHXClearWorkedAreaEvent.lua")
g_thMain:addSpecialization("vehicle", "vhxWorkedArea",      "VHXWorkedArea",    "workArea", "scripts/vehicles/specializations/VHXWorkedArea.lua")
g_thMain:addSpecialization("vehicle", "vhxFillTypeUsage",   "VHXFillTypeUsage", "fillUnit", "scripts/vehicles/specializations/VHXFillTypeUsage.lua")
g_thMain:addSpecialization("vehicle", "vhxCutter",          "VHXCutter",        "cutter",   "scripts/vehicles/specializations/VHXCutter.lua")
g_thMain:addSpecialization("vehicle", "vhxCombine",         "VHXCombine",       "combine",  "scripts/vehicles/specializations/VHXCombine.lua")
end
g_thMain:call(runScript)
end