-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXFillLevelsDisplay = {}
function VHXFillLevelsDisplay:initializeDataTable()
self.actionEventIds = {}
self.hasUsage = false
self.usageIsVisible = true
self.fillTypeTextSize = 0
return true
end
function VHXFillLevelsDisplay:setEnvironmentHooks()
local display = self:getParent()
g_thMain:setProtectedHook(display, "updateFillLevelFrames", VHXFillLevelsDisplay)
if g_thUnitConvert == nil then
g_thMain:setProtectedHook(display, "storeScaledValues",     VHXFillLevelsDisplay)
g_thMain:setProtectedHook(display, "draw",                  VHXFillLevelsDisplay)
display:storeScaledValues()
end
return true
end
function VHXFillLevelsDisplay:deleteDataTable()
self:restoreDefaultValue("updateFillLevelFrames")
if g_thUnitConvert == nil then
self:restoreDefaultValue("storeScaledValues")
self:restoreDefaultValue("draw")
end
self.hasEnvironmentHooks = false
return true
end
function VHXFillLevelsDisplay:registerActionEvents()
if g_thMain.isClient then
self:unregisterActionEvents()
local contextName = Vehicle.INPUT_CONTEXT_NAME
local actionEventIds = self.actionEventIds
g_inputBinding:beginActionEventsModification(contextName)
local _, eventId = g_inputBinding:registerActionEvent(InputAction.TH_TOGGLE_VHX_USAGE, self, self.onToggleUsage, false, true, false, true)
g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERY_LOW)
actionEventIds.toggleUsage = eventId
_, eventId = g_inputBinding:registerActionEvent(InputAction.TH_CLEAR_VHX_USAGE, self, self.onClearUsage, false, true, false, true)
g_inputBinding:setActionEventText(eventId, g_i18n:getText("thAction_clearVHXUsage"))
g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERY_LOW)
actionEventIds.clearUsage = eventId
g_inputBinding:endActionEventsModification()
end
end
function VHXFillLevelsDisplay:unregisterActionEvents()
if g_thMain.isClient then
for _, eventId in pairs(self.actionEventIds) do
g_inputBinding:removeActionEvent(eventId)
end
THUtils.clearTable(self.actionEventIds)
end
end
function VHXFillLevelsDisplay:updateActionEvents()
local display = self:getParent()
if g_thMain.isClient then
local vehicleData, vehicle = g_thMain:getDataTable(display.vehicle)
local actionEventIds = self.actionEventIds
local canToggleUsage = false
local canClearUsage = false
if display:getVisible() and vehicleData ~= nil and vehicle.getIsActiveForInput ~= nil then
if vehicle:getIsActiveForInput(true, true) then
canToggleUsage = self.hasUsage
canClearUsage = self.hasUsage and self.usageIsVisible
end
end
if actionEventIds.toggleUsage ~= nil then
local toggleUsageText = nil
if self.usageIsVisible then
toggleUsageText = g_i18n:getText("thAction_hideVHXUsage")
else
toggleUsageText = g_i18n:getText("thAction_showVHXUsage")
end
g_inputBinding:setActionEventText(actionEventIds.toggleUsage, toggleUsageText)
g_inputBinding:setActionEventActive(actionEventIds.toggleUsage, canToggleUsage)
g_inputBinding:setActionEventTextVisibility(actionEventIds.toggleUsage, canToggleUsage)
end
if actionEventIds.clearUsage ~= nil then
g_inputBinding:setActionEventActive(actionEventIds.clearUsage, canClearUsage)
g_inputBinding:setActionEventTextVisibility(actionEventIds.clearUsage, canClearUsage)
end
end
end
function VHXFillLevelsDisplay:getFillTypeUsageText(fillType, precision)
local display = self:getParent()
local vehicleData = g_thMain:getDataTable(display.vehicle)
local fillTypeInfo = THUtils.getFillType(fillType)
if vehicleData ~= nil and fillTypeInfo ~= nil then
local fillTypeIndex = fillTypeInfo.index
local usageValue, yieldValue = vehicleData:getVHXUsageValues(fillTypeIndex)
if yieldValue ~= nil then
usageValue = yieldValue
end
if usageValue ~= nil then
if g_thUnitConvert == nil then
return g_thMain:formatFillTypeUsage(usageValue, precision, fillTypeIndex)
end
local useAcres = g_gameSettings:getValue(GameSettings.SETTING.USE_ACRE)
local areaText, unitText = nil,nil
usageValue = usageValue * 10000
if useAcres then
areaText = g_i18n:getText("unit_acreShort")
usageValue = usageValue / THMain.ACRE_FACTOR
else
areaText = g_i18n:getText("unit_haShort")
end
local ucDisplayData = g_thUnitConvert:getDataTable(display)
local currentUnit, maxNumUnits = nil,nil
if ucDisplayData ~= nil then
currentUnit, maxNumUnits = ucDisplayData:getUnitSelectionValues()
end
local unitInfo, unitFactor = g_thUnitConvert:getFillTypeUnitData(fillTypeIndex, currentUnit, maxNumUnits)
if unitInfo ~= nil then
usageValue = usageValue * unitFactor
precision  = unitInfo.precision
unitText   = unitInfo.titleShort
else
precision = THUtils.getNoNil(precision, 0)
precision = math.floor(math.max(0, precision))
unitText  = fillTypeInfo.unitShort
if unitText == nil then
unitText = g_i18n:getVolumeUnit()
end
end
if precision > 0 then
return string.format("%0."..tostring(precision).."f %s/%s", usageValue, unitText, areaText)
end
return string.format("%d %s/%s", usageValue, unitText, areaText)
end
end
end
function VHXFillLevelsDisplay:onToggleUsage()
local function protectedFunc()
if self.hasUsage then
self.usageIsVisible = not self.usageIsVisible
end
end
g_thMain:call(protectedFunc)
end
function VHXFillLevelsDisplay:onClearUsage()
local function protectedFunc()
local display = self:getParent()
local vehicleData = g_thMain:getDataTable(display.vehicle)
if self.hasUsage and self.usageIsVisible and vehicleData ~= nil then
vehicleData:clearVHXUsage()
end
end
g_thMain:call(protectedFunc)
end
function VHXFillLevelsDisplay:hook_update(superFunc, dt, ...)
local envData = g_thMain:call("getDataTable", self, nil, true, VHXFillLevelsDisplay)
local function prependFunc()
if envData ~= nil then
if self.vehicle ~= envData.lastVehicle then
if self.vehicle == nil then
envData:unregisterActionEvents()
else
envData:registerActionEvents()
end
envData.lastVehicle = self.vehicle
end
envData.hasUsage = false
end
end
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
envData:updateActionEvents()
end
end
g_thMain:call(protectedChunk)
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, dt, ...))
end
function VHXFillLevelsDisplay:hook_updateFillLevelFrames(superFunc, ...)
local envData = g_thMain:call("getDataTable", self)
local fillTypeTitles = {}
local function prependFunc()
if envData ~= nil then
local vehicleData = g_thMain:getDataTable(self.vehicle)
local fillLevelBuffer = self.fillLevelBuffer
if vehicleData ~= nil and fillLevelBuffer ~= nil then
for _, bufferInfo in pairs(fillLevelBuffer) do
local fillTypeInfo = THUtils.getFillType(bufferInfo.fillType)
if fillTypeInfo ~= nil then
if fillTypeTitles[fillTypeInfo] == nil then
fillTypeTitles[fillTypeInfo] = fillTypeInfo.title
end
local usageText = envData:getFillTypeUsageText(fillTypeInfo.index, bufferInfo.precision)
if usageText ~= nil then
envData.hasUsage = true
if envData.usageIsVisible then
fillTypeInfo.title = usageText
end
end
end
end
end
end
end
local function appendFunc(...)
for fillTypeInfo, title in pairs(fillTypeTitles) do
fillTypeInfo.title = title
end
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, ...))
end
function VHXFillLevelsDisplay:hook_storeScaledValues(superFunc, ...)
local envData = g_thMain:call("getDataTable", self)
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
local defaultTextSize = HUDElement.TEXT_SIZE.DEFAULT_TEXT
local fillTypeTextSize = defaultTextSize - 3
local fillLevelTextSize = defaultTextSize - 1
local fillTypeTextPosX, fillTypeTextPosY = THUtils.unpack(FillLevelsDisplay.POSITION.FILL_TYPE_TEXT)
local fillTypeTextPos = {fillTypeTextPosX, fillTypeTextPosY - 9}
local fillLevelTextPosX, fillLevelTextPosY = THUtils.unpack(FillLevelsDisplay.POSITION.FILL_LEVEL_TEXT)
local fillLevelTextPos = {fillLevelTextPosX, fillLevelTextPosY - 2}
envData.fillTypeTextSize = self:scalePixelToScreenHeight(fillTypeTextSize)
self.fillLevelTextSize = self:scalePixelToScreenHeight(fillLevelTextSize)
self.fillTypeTextOffsetX, self.fillTypeTextOffsetY = self:scalePixelToScreenVector(fillTypeTextPos)
self.fillLevelTextOffsetX, self.fillLevelTextOffsetY = self:scalePixelToScreenVector(fillLevelTextPos)
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
function VHXFillLevelsDisplay:hook_draw(superFunc, ...)
local envData = g_thMain:call("getDataTable", self)
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
local fillLevelTextBuffer = self.fillLevelTextBuffer
local numFillLevelEntries = #fillLevelTextBuffer
if self:getVisible() and numFillLevelEntries > 0 then
local fillTypeTextBuffer = self.fillTypeTextBuffer
local baseX, baseY = self:getPosition()
local width = self:getWidth()
for i = 1, numFillLevelEntries do
local fillTypeText = fillTypeTextBuffer[i]
if fillTypeText ~= nil then
local posX = baseX + width + self.fillTypeTextOffsetX
local posY = baseY + (i - 1) * (self.frameHeight + self.frameOffsetY)
if i == 1 then
posX = posX + self.firstFillTypeOffset
end
setTextColor(THUtils.unpack(FillLevelsDisplay.COLOR.FILL_LEVEL_TEXT))
setTextAlignment(RenderText.ALIGN_RIGHT)
setTextBold(false)
renderText(posX, posY + self.fillTypeTextOffsetY, envData.fillTypeTextSize, fillTypeText)
end
end
end
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
local function runScript()
g_thMain:setProtectedHook("FillLevelsDisplay", "update", VHXFillLevelsDisplay)
end
g_thMain:call(runScript)