-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXSpeedMeterDisplay = {}
function VHXSpeedMeterDisplay:initializeDataTable()
self.actionEventIds = {}
self.workedAreaTextPosX = 0
self.workedAreaTextPosY = 0
self.workedAreaTextOffsetX = 0
self.workedAreaTextOffsetY = 0
self.workedAreaTextSize = 0
self.hasWorkedArea = false
return true
end
function VHXSpeedMeterDisplay:registerActionEvents()
if g_thMain.isClient then
self:unregisterActionEvents()
local contextName = Vehicle.INPUT_CONTEXT_NAME
local actionEventIds = self.actionEventIds
g_inputBinding:beginActionEventsModification(contextName)
local _, eventId = g_inputBinding:registerActionEvent(InputAction.TH_CLEAR_VHX_WORKED_AREA, self, self.onClearWorkedArea, false, true, false, true)
g_inputBinding:setActionEventText(eventId, g_i18n:getText("thAction_clearVHXWorkedArea"))
g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERY_LOW)
actionEventIds.clearWorkedArea = eventId
g_inputBinding:endActionEventsModification()
end
end
function VHXSpeedMeterDisplay:unregisterActionEvents()
if g_thMain.isClient then
for _, eventId in pairs(self.actionEventIds) do
g_inputBinding:removeActionEvent(eventId)
end
THUtils.clearTable(self.actionEventIds)
end
end
function VHXSpeedMeterDisplay:updateActionEvents()
local display = self:getParent()
if g_thMain.isClient then
local vehicleData, vehicle = g_thMain:getDataTable(display.vehicle)
local actionEventIds = self.actionEventIds
local canClearWorkedArea = false
if display:getVisible() and vehicleData ~= nil and vehicle.getIsActiveForInput ~= nil then
if vehicle:getIsActiveForInput(true, true) then
canClearWorkedArea = self.hasWorkedArea
end
end
if actionEventIds.clearWorkedArea ~= nil then
g_inputBinding:setActionEventActive(actionEventIds.clearWorkedArea, canClearWorkedArea)
g_inputBinding:setActionEventTextVisibility(actionEventIds.clearWorkedArea, canClearWorkedArea)
end
end
end
function VHXSpeedMeterDisplay:updateWorkedArea(dt)
local display = self:getParent()
local vehicleData, vehicle = g_thMain:getDataTable(display.vehicle)
local gaugeBackgroundElement = display.gaugeBackgroundElement
local operatingTimeElement = display.operatingTimeElement
self.hasWorkedArea = false
self.workedAreaText = nil
if display:getVisible() and gaugeBackgroundElement ~= nil and operatingTimeElement ~= nil then
if vehicleData ~= nil then
local workedAreaValue = vehicleData:getVHXWorkedArea()
if workedAreaValue ~= nil then
self.hasWorkedArea = true
local posX = gaugeBackgroundElement:getPosition()
local _, posY = operatingTimeElement:getPosition()
local elementWidth = gaugeBackgroundElement:getWidth()
local elementHeight = operatingTimeElement:getHeight()
local workedAreaText = g_thMain:formatWorkedArea(workedAreaValue)
self.workedAreaTextPosX = posX + (elementWidth / 2) + self.workedAreaTextOffsetX
self.workedAreaTextPosY = posY + elementHeight + self.workedAreaTextOffsetY
self.workedAreaText = workedAreaText
end
end
end
end
function VHXSpeedMeterDisplay:drawWorkedAreaText()
setTextAlignment(RenderText.ALIGN_CENTER)
setTextBold(true)
setTextColor(THUtils.unpack(VHXSpeedMeterDisplay.COLOR.WORKED_AREA_TEXT))
renderText(self.workedAreaTextPosX, self.workedAreaTextPosY, self.workedAreaTextSize, self.workedAreaText)
end
function VHXSpeedMeterDisplay:onClearWorkedArea()
local function protectedFunc()
local display = self:getParent()
local vehicleData = g_thMain:getDataTable(display.vehicle)
if self.hasWorkedArea and vehicleData ~= nil then
vehicleData:clearVHXWorkedArea()
end
end
g_thMain:call(protectedFunc)
end
function VHXSpeedMeterDisplay:hook_update(superFunc, dt, ...)
local envData = g_thMain:call("getDataTable", self, nil, true, VHXSpeedMeterDisplay)
local function prependFunc()
if envData ~= nil then
if self.vehicle ~= envData.lastVehicle then
if self.vehicle == nil then
envData:unregisterActionEvents()
else
envData:registerActionEvents()
end
envData.lastVehicle = self.vehicle
end
if self.vehicle == nil then
envData.selectedVehicle = nil
else
local selectedVehicle = self.vehicle:getSelectedVehicle()
envData.selectedVehicle = THUtils.getNoNil(selectedVehicle, self.vehicle)
end
end
end
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
envData:updateWorkedArea(dt)
envData:updateActionEvents()
end
end
g_thMain:call(protectedChunk)
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, dt, ...))
end
function VHXSpeedMeterDisplay:hook_draw(superFunc, ...)
local envData = g_thMain:call("getDataTable", self)
local function appendFunc(...)
if envData ~= nil then
if self.isVehicleDrawSafe and self:getVisible() then
if envData.workedAreaText ~= nil then
envData:drawWorkedAreaText()
end
end
end
end
return appendFunc(superFunc(self, ...))
end
function VHXSpeedMeterDisplay:hook_storeScaledValues(superFunc, baseX, baseY, ...)
local envData = g_thMain:call("getDataTable", self)
local function appendFunc(...)
local function protectedChunk()
if envData ~= nil then
envData.workedAreaTextOffsetX, envData.workedAreaTextOffsetY = self:scalePixelToScreenVector(VHXSpeedMeterDisplay.POSITION.WORKED_AREA_TEXT)
envData.workedAreaTextSize = self:scalePixelToScreenHeight(VHXSpeedMeterDisplay.TEXT_SIZE.WORKED_AREA)
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, baseX, baseY, ...))
end
local function runScript()
local workedAreaTextR, workedAreaTextG, workedAreaTextB = THUtils.unpack(SpeedMeterDisplay.COLOR.DAMAGE_GAUGE)
VHXSpeedMeterDisplay.COLOR = {
WORKED_AREA_TEXT = {workedAreaTextR, workedAreaTextG, workedAreaTextB, 1}
}
VHXSpeedMeterDisplay.POSITION = {
WORKED_AREA_TEXT = {0, 2}
}
VHXSpeedMeterDisplay.TEXT_SIZE = {
WORKED_AREA = SpeedMeterDisplay.TEXT_SIZE.OPERATING_TIME - 2
}
g_thMain:setProtectedHook("SpeedMeterDisplay", "draw",              VHXSpeedMeterDisplay)
g_thMain:setProtectedHook("SpeedMeterDisplay", "update",            VHXSpeedMeterDisplay)
g_thMain:setProtectedHook("SpeedMeterDisplay", "storeScaledValues", VHXSpeedMeterDisplay)
end
g_thMain:call(runScript)