-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXCombine = {}
function VHXCombine.prerequisitesPresent(specializations)
if SpecializationUtil.hasSpecialization(VHXFillTypeUsage, specializations)
and SpecializationUtil.hasSpecialization(Combine, specializations)
then
return true
end
return false
end
function VHXCombine.registerEventListeners(vehicleType)
SpecializationUtil.registerEventListener(vehicleType, "onLoad", VHXCombine)
end
function VHXCombine.registerOverwrittenFunctions(vehicleType)
SpecializationUtil.registerOverwrittenFunction(vehicleType, "addCutterArea", VHXCombine.addCutterArea)
end
function VHXCombine:onLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxCombine")
specTable.yieldFillUnits = {}
specTable.yieldFillTypes = {}
specTable.usageFillUnits = {}
specTable.usageFillTypes = {}
end
function VHXCombine:addCutterArea(superFunc, area, realArea, inputFruitType, outputFillType, strawRatio, ...)
local specTable = g_thMain:call("getSpecTable", self, "vhxCombine")
local vhxUsageSpec = g_thMain:call("getSpecTable", self, "vhxFillTypeUsage")
local combineSpec = self.spec_combine
local _, addFillUnitFillLevelFunc, oldAddFillUnitFillLevelFunc = nil,nil,nil
local lastWorkedArea = nil
local isUsageDirty = false
if type(area) == "table" then
lastWorkedArea = area.realValue
area = area.value
end
local function prependFunc()
if specTable ~= nil then
THUtils.clearTable(specTable.usageFillUnits)
THUtils.clearTable(specTable.usageFillTypes)
THUtils.clearTable(specTable.yieldFillUnits)
THUtils.clearTable(specTable.yieldFillTypes)
if self.isServer and lastWorkedArea ~= nil and vhxUsageSpec ~= nil and combineSpec ~= nil then
local function hook_addFillUnitFillLevel(pSuperFunc, pSelf, pFarmId, pFillUnitIndex, pFillLevelDelta, pFillTypeIndex, ...)
local function prependFunc2()
if pFillUnitIndex ~= nil and pFillTypeIndex ~= nil and pFillTypeIndex ~= FillType.UNKNOWN then
if pFillLevelDelta ~= nil then
local fillLevel = self:getFillUnitFillLevel(pFillUnitIndex)
local isAdditive = false
fillLevel = THUtils.getNoNil(fillLevel, 0)
if combineSpec.additives ~= nil and combineSpec.additives.fillUnitIndex ~= nil then
if pFillUnitIndex == combineSpec.additives.fillUnitIndex then
if pFillLevelDelta <= 0 then
specTable.usageFillUnits[pFillUnitIndex] = {fillType = pFillTypeIndex, fillLevel = fillLevel}
specTable.usageFillTypes[pFillTypeIndex] = true
isUsageDirty = true
isAdditive = true
end
end
end
if not isAdditive then
specTable.yieldFillUnits[pFillUnitIndex] = {fillType = pFillTypeIndex, fillLevel = fillLevel}
specTable.yieldFillTypes[pFillTypeIndex] = true
isUsageDirty = true
end
end
end
end
g_thMain:call(prependFunc2)
return pSuperFunc(pSelf, pFarmId, pFillUnitIndex, pFillLevelDelta, pFillTypeIndex, ...)
end
_, addFillUnitFillLevelFunc, oldAddFillUnitFillLevelFunc = THUtils.hookFunction(self, "addFillUnitFillLevel", hook_addFillUnitFillLevel)
end
end
end
local function appendFunc(...)
if addFillUnitFillLevelFunc ~= nil then rawset(self, "addFillUnitFillLevel", oldAddFillUnitFillLevelFunc) end
local function protectedChunk()
if isUsageDirty then
local areaWorked = 0
if g_currentMission ~= nil then
areaWorked = lastWorkedArea * g_currentMission:getFruitPixelsToSqm()
end
if areaWorked > 0 then
for fillUnitIndex, fillUnitEntry in pairs(specTable.usageFillUnits) do
local fillTypeIndex = self:getFillUnitFillType(fillUnitIndex)
if fillTypeIndex == fillUnitEntry.fillType then
local oldFillLevel = fillUnitEntry.fillLevel
local fillLevel = self:getFillUnitFillLevel(fillUnitIndex)
if fillLevel ~= nil and fillLevel >= 0 then
local deltaFill = math.max(0, oldFillLevel - fillLevel)
local usageValue = deltaFill / areaWorked
vhxUsageSpec:updateVHXUsage(usageValue, fillTypeIndex, false)
end
end
end
for fillUnitIndex, fillUnitEntry in pairs(specTable.yieldFillUnits) do
local fillTypeIndex = self:getFillUnitFillType(fillUnitIndex)
if fillTypeIndex == fillUnitEntry.fillType then
local oldFillLevel = fillUnitEntry.fillLevel
local fillLevel = self:getFillUnitFillLevel(fillUnitIndex)
if fillLevel ~= nil and fillLevel >= 0 then
local deltaFill = math.max(0, fillLevel - oldFillLevel)
local yieldValue = deltaFill / areaWorked
vhxUsageSpec:updateVHXUsage(yieldValue, fillTypeIndex, true)
end
end
end
end
end
end
g_thMain:call(protectedChunk)
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, area, realArea, inputFruitType, outputFillType, strawRatio, ...))
end