-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXCutter = {}
function VHXCutter.prerequisitesPresent(specializations)
return SpecializationUtil.hasSpecialization(Cutter, specializations)
end
function VHXCutter.registerEventListeners(vehicleType)
SpecializationUtil.registerEventListener(vehicleType, "onLoad", VHXCutter)
end
function VHXCutter.registerOverwrittenFunctions(vehicleType)
SpecializationUtil.registerOverwrittenFunction(vehicleType, "processCutterArea", VHXCutter.processCutterArea)
end
function VHXCutter:onLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxCutter")
specTable.lastCutterArea = {}
end
function VHXCutter:processCutterArea(superFunc, ...)
local specTable = g_thMain:call("getSpecTable", self, "vhxCutter")
local densityMapUtil, cutFruitAreaFunc, oldCutFruitAreaFunc = nil,nil,nil
local function prependFunc()
if specTable ~= nil then
if self.isServer then
local function hook_cutFruitArea(pSuperFunc, pFruitTypeIndex, ...)
local function appendFunc2(pRealArea, ...)
local function protectedChunk2()
if pFruitTypeIndex ~= nil and pFruitTypeIndex ~= FruitType.UNKNOWN then
if pRealArea ~= nil and pRealArea >= 0 then
local lastCutterArea = THUtils.getNoNil(specTable.lastCutterArea[pFruitTypeIndex], 0)
lastCutterArea = lastCutterArea + pRealArea
specTable.lastCutterArea[pFruitTypeIndex] = lastCutterArea
end
end
end
g_thMain:call(protectedChunk2)
return pRealArea, ...
end
return appendFunc2(pSuperFunc(pFruitTypeIndex, ...))
end
densityMapUtil, cutFruitAreaFunc, oldCutFruitAreaFunc = THUtils.hookFunction("FSDensityMapUtil", "cutFruitArea", hook_cutFruitArea)
end
end
end
local function appendFunc(...)
if cutFruitAreaFunc ~= nil then rawset(densityMapUtil, "cutFruitArea", oldCutFruitAreaFunc) end
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, ...))
end
function VHXCutter:hook_onStartWorkAreaProcessing(superFunc, ...)
local specTable = g_thMain:call("getSpecTable", self, "vhxCutter")
local function prependFunc()
if specTable ~= nil then
THUtils.clearTable(specTable.lastCutterArea)
end
end
g_thMain:call(prependFunc)
return superFunc(self, ...)
end
function VHXCutter:hook_onEndWorkAreaProcessing(superFunc, ...)
local specTable = g_thMain:call("getSpecTable", self, "vhxCutter")
local cutterSpec = self.spec_cutter
local combineVehicle, addCutterAreaFunc, oldAddCutterAreaFunc = nil,nil,nil
local function prependFunc()
if specTable ~= nil then
if self.isServer and cutterSpec ~= nil and cutterSpec.workAreaParameters ~= nil then
combineVehicle = cutterSpec.workAreaParameters.combineVehicle
if combineVehicle ~= nil then
local vhxCombineSpec = g_thMain:getSpecTable(combineVehicle, "vhxCombine")
if vhxCombineSpec ~= nil then
local function hook_addCutterArea(pSuperFunc, pSelf, pLastArea, pLastRealArea, pInputFruitType, pOutputFillType, ...)
local function prependFunc2()
if pInputFruitType ~= nil and pInputFruitType ~= FruitType.UNKNOWN then
local lastCutterArea = specTable.lastCutterArea[pInputFruitType]
if lastCutterArea ~= nil and lastCutterArea >= 0 then
local areaValue = pLastArea
pLastArea = {value = areaValue, realValue = lastCutterArea}
end
end
end
g_thMain:call(prependFunc2)
return pSuperFunc(pSelf, pLastArea, pLastRealArea, pInputFruitType, pOutputFillType, ...)
end
_, addCutterAreaFunc, oldAddCutterAreaFunc = THUtils.hookFunction(combineVehicle, "addCutterArea", hook_addCutterArea)
end
end
end
end
end
local function appendFunc(...)
if addCutterAreaFunc ~= nil then rawset(combineVehicle, "addCutterArea", oldAddCutterAreaFunc) end
return ...
end
g_thMain:call(prependFunc)
return appendFunc(superFunc(self, ...))
end
local function runScript()
g_thMain:setProtectedHook("Cutter", "onStartWorkAreaProcessing", VHXCutter)
g_thMain:setProtectedHook("Cutter", "onEndWorkAreaProcessing",   VHXCutter)
end
g_thMain:call(runScript)