-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXFillTypeUsage = {}
VHXFillTypeUsage.SEND_NUM_BITS = 32
function VHXFillTypeUsage.initSpecialization()
local schemaSavegame = Vehicle.xmlSchemaSavegame
local savegameBasePath = "vehicles.vehicle(?)."..g_thMain.modName..".vhxFillTypeUsage"
local savegameUsagePath = savegameBasePath..".usageEntries.usageEntry(?)"
schemaSavegame:register(XMLValueType.STRING, savegameUsagePath.."#fillType",   "Entry fillType name")
schemaSavegame:register(XMLValueType.FLOAT,  savegameUsagePath.."#totalValue", "Total usage value")
schemaSavegame:register(XMLValueType.INT,    savegameUsagePath.."#numValues",  "Number of saved values")
local savegameYieldPath = savegameBasePath..".yieldEntries.yieldEntry(?)"
schemaSavegame:register(XMLValueType.STRING, savegameYieldPath.."#fillType",   "Entry fillType name")
schemaSavegame:register(XMLValueType.FLOAT,  savegameYieldPath.."#totalValue", "Total usage value")
schemaSavegame:register(XMLValueType.INT,    savegameYieldPath.."#numValues",   "Number of saved values")
end
function VHXFillTypeUsage.prerequisitesPresent(specializations)
return SpecializationUtil.hasSpecialization(FillUnit, specializations)
end
function VHXFillTypeUsage.registerEventListeners(vehicleType)
SpecializationUtil.registerEventListener(vehicleType, "onLoad",              VHXFillTypeUsage)
SpecializationUtil.registerEventListener(vehicleType, "onPostLoad",          VHXFillTypeUsage)
SpecializationUtil.registerEventListener(vehicleType, "onReadStream",        VHXFillTypeUsage)
SpecializationUtil.registerEventListener(vehicleType, "onWriteStream",       VHXFillTypeUsage)
SpecializationUtil.registerEventListener(vehicleType, "onReadUpdateStream",  VHXFillTypeUsage)
SpecializationUtil.registerEventListener(vehicleType, "onWriteUpdateStream", VHXFillTypeUsage)
end
function VHXFillTypeUsage:onLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
specTable.usageEntries = {}
specTable.usageMapping = {}
specTable.yieldEntries = {}
specTable.yieldMapping = {}
THUtils.registerSpecFunction(self, specTable, "updateVHXUsage",  VHXFillTypeUsage)
THUtils.registerSpecFunction(self, specTable, "clearVHXUsage",   VHXFillTypeUsage)
THUtils.registerSpecFunction(self, specTable, "getVHXUsageData", VHXFillTypeUsage)
specTable.dirtyFlag = self:getNextDirtyFlag()
end
function VHXFillTypeUsage:onPostLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
if self.isServer and savegame ~= nil then
local xmlFile = savegame.xmlFile
local baseKey = savegame.key.."."..g_thMain.modName..".vhxFillTypeUsage"
if xmlFile:hasProperty(baseKey) then
xmlFile:iterate(baseKey..".usageEntries.usageEntry", function(pIndex, pKey)
local fillType   = xmlFile:getValue(pKey.."#fillType")
local totalValue = xmlFile:getValue(pKey.."#totalValue")
local numValues  = xmlFile:getValue(pKey.."#numValues")
local fillTypeInfo = THUtils.getFillType(fillType)
if fillTypeInfo ~= nil and totalValue ~= nil then
local usageTable = specTable:updateVHXUsage(totalValue, fillType, false)
if usageTable ~= nil then
numValues = THUtils.getNoNil(numValues, 1)
usageTable.numValues = math.floor(math.max(1, numValues))
usageTable.average = usageTable.total / usageTable.numValues
end
end
end)
xmlFile:iterate(baseKey..".yieldEntries.yieldEntry", function(pIndex, pKey)
local fillType   = xmlFile:getValue(pKey.."#fillType")
local totalValue = xmlFile:getValue(pKey.."#totalValue")
local numValues  = xmlFile:getValue(pKey.."#numValues")
local fillTypeInfo = THUtils.getFillType(fillType)
if fillTypeInfo ~= nil and totalValue ~= nil then
local yieldTable = specTable:updateVHXUsage(totalValue, fillType, true)
if yieldTable ~= nil then
numValues = THUtils.getNoNil(numValues, 1)
yieldTable.numValues = math.floor(math.max(0, numValues))
yieldTable.average = yieldTable.total / yieldTable.numValues
end
end
end)
end
end
end
function VHXFillTypeUsage:saveToXMLFile(xmlFile, key, usedModNames)
local specTable = g_thMain:call("getSpecTable", self, "vhxFillTypeUsage")
local function protectedFunc()
if specTable ~= nil then
if self.isServer then
if #specTable.usageEntries > 0 then
local usageIdx = 0
for _, usageTable in ipairs(specTable.usageEntries) do
local usageKey = string.format("%s.usageEntries.usageEntry(%d)", key, usageIdx)
local fillTypeInfo = THUtils.getFillType(usageTable.fillType, true, true)
local fillTypeName = fillTypeInfo.name
if fillTypeName ~= nil and usageTable.total > 0 then
xmlFile:setValue(usageKey.."#fillType",   fillTypeName)
xmlFile:setValue(usageKey.."#totalValue", usageTable.total)
xmlFile:setValue(usageKey.."#numValues",  usageTable.numValues)
usageIdx = usageIdx + 1
end
end
end
if #specTable.yieldEntries > 0 then
local yieldIdx = 0
for _, yieldTable in ipairs(specTable.yieldEntries) do
local yieldKey = string.format("%s.yieldEntries.yieldEntry(%d)", key, yieldIdx)
local fillTypeInfo = THUtils.getFillType(yieldTable.fillType, true, true)
local fillTypeName = fillTypeInfo.name
if fillTypeName ~= nil and yieldTable.total > 0 then
xmlFile:setValue(yieldKey.."#fillType",   fillTypeName)
xmlFile:setValue(yieldKey.."#totalValue", yieldTable.total)
xmlFile:setValue(yieldKey.."#numValues",  yieldTable.numValues)
yieldIdx = yieldIdx + 1
end
end
end
end
end
end
g_thMain:call(protectedFunc)
end
function VHXFillTypeUsage:onReadStream(streamId, connection)
if connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
local numUsageEntries = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local numYieldEntries = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
if streamReadBool(streamId) then
for i = 1, numUsageEntries do
local fillTypeIndex = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local usageValue = streamReadFloat32(streamId)
local isActive = streamReadBool(streamId)
local usageTable = specTable:updateVHXUsage(usageValue, fillTypeIndex, false)
if usageTable ~= nil then
usageTable.isActive = isActive
end
end
end
if streamReadBool(streamId) then
for i = 1, numYieldEntries do
local fillTypeIndex = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local yieldValue = streamReadFloat32(streamId)
local isActive = streamReadBool(streamId)
local yieldTable = specTable:updateVHXUsage(yieldValue, fillTypeIndex, true)
if yieldTable ~= nil then
yieldTable.isActive = isActive
end
end
end
end
end
function VHXFillTypeUsage:onWriteStream(streamId, connection)
if not connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
local numUsageEntries = #specTable.usageEntries
local numYieldEntries = #specTable.yieldEntries
streamWriteUIntN(streamId, numUsageEntries, FillTypeManager.SEND_NUM_BITS)
streamWriteUIntN(streamId, numYieldEntries, FillTypeManager.SEND_NUM_BITS)
if streamWriteBool(streamId, numUsageEntries > 0) then
for i = 1, numUsageEntries do
local usageTable = specTable.usageEntries[i]
streamWriteUIntN(streamId, usageTable.fillType, FillTypeManager.SEND_NUM_BITS)
streamWriteFloat32(streamId, usageTable.average)
streamWriteBool(streamId, usageTable.isActive)
end
end
if streamWriteBool(streamId, numYieldEntries > 0) then
for i = 1, numYieldEntries do
local yieldTable = specTable.yieldEntries[i]
streamWriteUIntN(streamId, yieldTable.fillType, FillTypeManager.SEND_NUM_BITS)
streamWriteFloat32(streamId, yieldTable.average)
streamWriteBool(streamId, yieldTable.isActive)
end
end
end
end
function VHXFillTypeUsage:onReadUpdateStream(streamId, timeStamp, connection)
if connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
if streamReadBool(streamId) then
local numUsageEntries = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local numYieldEntries = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
if streamReadBool(streamId) then
for i = 1, numUsageEntries do
local fillTypeIndex = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local usageValue = streamReadFloat32(streamId)
local isActive = streamReadBool(streamId)
local usageTable = specTable:updateVHXUsage(usageValue, fillTypeIndex, false)
if usageTable ~= nil then
usageTable.isActive = isActive
end
end
end
if streamReadBool(streamId) then
for i = 1, numYieldEntries do
local fillTypeIndex = streamReadUIntN(streamId, FillTypeManager.SEND_NUM_BITS)
local yieldValue = streamReadFloat32(streamId)
local isActive = streamReadBool(streamId)
local yieldTable = specTable:updateVHXUsage(yieldValue, fillTypeIndex, true)
if yieldTable ~= nil then
yieldTable.isActive = isActive
end
end
end
end
end
end
function VHXFillTypeUsage:onWriteUpdateStream(streamId, connection, dirtyMask)
if not connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
if streamWriteBool(streamId, bitAND(dirtyMask, specTable.dirtyFlag) ~= 0) then
local numUsageEntries = #specTable.usageEntries
local numYieldEntries = #specTable.yieldEntries
streamWriteUIntN(streamId, numUsageEntries, FillTypeManager.SEND_NUM_BITS)
streamWriteUIntN(streamId, numYieldEntries, FillTypeManager.SEND_NUM_BITS)
if streamWriteBool(streamId, numUsageEntries > 0) then
for i = 1, numUsageEntries do
local usageTable = specTable.usageEntries[i]
streamWriteUIntN(streamId, usageTable.fillType, FillTypeManager.SEND_NUM_BITS)
streamWriteFloat32(streamId, usageTable.average)
streamWriteBool(streamId, usageTable.isActive)
end
end
if streamWriteBool(streamId, numYieldEntries > 0) then
for i = 1, numYieldEntries do
local yieldTable = specTable.yieldEntries[i]
streamWriteUIntN(streamId, yieldTable.fillType, FillTypeManager.SEND_NUM_BITS)
streamWriteFloat32(streamId, yieldTable.average)
streamWriteBool(streamId, yieldTable.isActive)
end
end
end
end
end
function VHXFillTypeUsage:updateVHXUsage(value, fillType, isYield)
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
local fillTypeInfo = THUtils.getFillType(fillType, true, true)
if  fillTypeInfo ~= nil
and THUtils.argIsValid(type(value) == "number" and value >= 0, "value", value, true)
and THUtils.argIsValid(not isYield or isYield == true, "isYield", isYield, true)
then
local fillTypeIndex = fillTypeInfo.index
local usageEntries, usageMapping = nil,nil
if isYield then
usageEntries = specTable.yieldEntries
usageMapping = specTable.yieldMapping
else
usageEntries = specTable.usageEntries
usageMapping = specTable.usageMapping
end
local usageIndex = usageMapping[fillTypeIndex]
local usageTable = nil
if usageIndex ~= nil then
usageTable = usageEntries[usageIndex]
if usageTable == nil then
THUtils.errorMsg(true, "FillType %q mapped to non-existant usage table", fillTypeInfo.name)
return
end
end
if usageTable == nil then
if fillTypeIndex ~= FillType.UNKNOWN then
usageTable = {
total     = 0,
average   = 0,
numValues = 0,
fillType  = fillTypeIndex,
isActive  = true
}
table.insert(usageEntries, usageTable)
usageMapping[fillTypeIndex] = #usageEntries
end
end
if usageTable ~= nil then
if self.isServer and usageTable.numValues > 0 then
usageTable.total = usageTable.total + value
usageTable.numValues = usageTable.numValues + 1
usageTable.average = usageTable.total / usageTable.numValues
local maxValue = (2 ^ THMain.MAX_VALUE_BITS) - 1
if usageTable.total > maxValue or usageTable.numValues > maxValue then
usageTable.total = usageTable.average
usageTable.numValues = 1
end
else
usageTable.total = value
usageTable.average = value
usageTable.numValues = 1
end
if self.isServer then
self:raiseDirtyFlags(specTable.dirtyFlag)
end
return usageTable
end
end
end
function VHXFillTypeUsage:clearVHXUsage()
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
if self.isServer then
for _, usageTable in pairs(specTable.usageEntries) do
usageTable.total = 0
usageTable.average = 0
usageTable.numValues = 0
end
for _, yieldTable in pairs(specTable.yieldEntries) do
yieldTable.total = 0
yieldTable.average = 0
yieldTable.numValues = 0
end
self:raiseDirtyFlags(specTable.dirtyFlag)
end
end
function VHXFillTypeUsage:getVHXUsageData(fillType)
local specTable = g_thMain:getSpecTable(self, "vhxFillTypeUsage")
local fillTypeInfo = THUtils.getFillType(fillType)
local usageTable, yieldTable = nil,nil
if fillTypeInfo ~= nil then
local usageIndex = specTable.usageMapping[fillTypeInfo.index]
local yieldIndex = specTable.yieldMapping[fillTypeInfo.index]
if usageIndex ~= nil then
usageTable = specTable.usageEntries[usageIndex]
end
if yieldIndex ~= nil then
yieldTable = specTable.yieldEntries[yieldIndex]
end
end
return usageTable, yieldTable
end