-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

VHXWorkedArea = {}
VHXWorkedArea.SEND_NUM_BITS = 32
function VHXWorkedArea.initSpecialization()
local schemaSavegame = Vehicle.xmlSchemaSavegame
local savegameBasePath = "vehicles.vehicle(?)."..g_thMain.modName..".vhxWorkedArea"
schemaSavegame:register(XMLValueType.FLOAT, savegameBasePath.."#value", "Total worked area")
end
function VHXWorkedArea.prerequisitesPresent(specializations)
return SpecializationUtil.hasSpecialization(WorkArea, specializations)
end
function VHXWorkedArea.registerEventListeners(vehicleType)
SpecializationUtil.registerEventListener(vehicleType, "onLoad",              VHXWorkedArea)
SpecializationUtil.registerEventListener(vehicleType, "onPostLoad",          VHXWorkedArea)
SpecializationUtil.registerEventListener(vehicleType, "onReadStream",        VHXWorkedArea)
SpecializationUtil.registerEventListener(vehicleType, "onWriteStream",       VHXWorkedArea)
SpecializationUtil.registerEventListener(vehicleType, "onReadUpdateStream",  VHXWorkedArea)
SpecializationUtil.registerEventListener(vehicleType, "onWriteUpdateStream", VHXWorkedArea)
end
function VHXWorkedArea.registerOverwrittenFunctions(vehicleType)
SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateLastWorkedArea", VHXWorkedArea.updateLastWorkedArea)
end
function VHXWorkedArea:onLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
specTable.workedHectares = 0
THUtils.registerSpecFunction(self, specTable, "updateVHXWorkedArea", VHXWorkedArea)
THUtils.registerSpecFunction(self, specTable, "clearVHXWorkedArea",  VHXWorkedArea)
THUtils.registerSpecFunction(self, specTable, "getVHXWorkedArea",    VHXWorkedArea)
specTable.dirtyFlag = self:getNextDirtyFlag()
end
function VHXWorkedArea:onPostLoad(savegame)
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
if self.isServer and savegame ~= nil then
local xmlFile = savegame.xmlFile
local baseKey = savegame.key.."."..g_thMain.modName..".vhxWorkedArea"
if xmlFile:hasProperty(baseKey) then
local value = xmlFile:getValue(baseKey.."#value")
if value ~= nil then
specTable.workedHectares = value
end
end
end
end
function VHXWorkedArea:saveToXMLFile(xmlFile, key, usedModNames)
local specTable = g_thMain:call("getSpecTable", self, "vhxWorkedArea")
local function protectedFunc()
if specTable ~= nil then
if self.isServer then
local baseKey = key
local workedHectares = specTable.workedHectares
if workedHectares ~= nil and workedHectares > 0 then
xmlFile:setValue(baseKey.."#value", workedHectares)
end
end
end
end
g_thMain:call(protectedFunc)
end
function VHXWorkedArea:onReadStream(streamId, connection)
if connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
specTable.workedHectares = streamReadFloat32(streamId)
end
end
function VHXWorkedArea:onWriteStream(streamId, connection)
if not connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
streamWriteFloat32(streamId, specTable.workedHectares)
end
end
function VHXWorkedArea:onReadUpdateStream(streamId, timeStamp, connection)
if connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
if streamReadBool(streamId) then
specTable.workedHectares = streamReadFloat32(streamId)
end
end
end
function VHXWorkedArea:onWriteUpdateStream(streamId, connection, dirtyMask)
if not connection:getIsServer() then
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
if streamWriteBool(streamId, bitAND(dirtyMask, specTable.dirtyFlag) ~= 0) then
streamWriteFloat32(streamId, specTable.workedHectares)
end
end
end
function VHXWorkedArea:updateVHXWorkedArea(value, isHa)
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
if THUtils.argIsValid(type(value) == "number" and value >= 0, "value", value, true) then
if self.isServer then
if isHa ~= true then
if g_currentMission ~= nil then
value = MathUtil.areaToHa(value, g_currentMission:getFruitPixelsToSqm())
else
value = 0
end
end
if value > 0 then
specTable.workedHectares = specTable.workedHectares + value
self:raiseDirtyFlags(specTable.dirtyFlag)
end
return specTable.workedHectares
end
end
end
function VHXWorkedArea:clearVHXWorkedArea()
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
if self.isServer then
specTable.workedHectares = 0
self:raiseDirtyFlags(specTable.dirtyFlag)
end
end
function VHXWorkedArea:getVHXWorkedArea()
local specTable = g_thMain:getSpecTable(self, "vhxWorkedArea")
return specTable.workedHectares
end
function VHXWorkedArea:updateLastWorkedArea(superFunc, area, ...)
local specTable = g_thMain:call("getSpecTable", self, "vhxWorkedArea")
local workAreaSpec = self.spec_workArea
local function appendFunc(...)
local function protectedChunk()
if specTable ~= nil then
if self.isServer then
local lastWorkedArea = THUtils.getNoNil(workAreaSpec.lastWorkedArea, 0)
if lastWorkedArea > 0 then
specTable:updateVHXWorkedArea(lastWorkedArea)
end
end
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, area, ...))
end